"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    convertLessError: function() {
        return _lessutils.convertLessError;
    },
    styleLoader: function() {
        return styleLoader;
    },
    transformLess: function() {
        return _transformless.transformLess;
    }
});
const _promises = require("node:fs/promises");
const _nodepath = /*#__PURE__*/ _interop_require_default(require("node:path"));
const _colors = /*#__PURE__*/ _interop_require_default(require("colors"));
const _deepmerge = /*#__PURE__*/ _interop_require_default(require("deepmerge"));
const _lightningcss = require("lightningcss");
const _querystring = /*#__PURE__*/ _interop_require_default(require("query-string"));
const _lessutils = require("./less-utils.cjs");
const _sassutils = require("./sass-utils.js");
const _transformless = require("./transform-less.js");
const _transformsass = require("./transform-sass.js");
const _utils = require("./utils.js");
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _object_spread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _define_property(target, key, source[key]);
        });
    }
    return target;
}
function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
        var symbols = Object.getOwnPropertySymbols(object);
        if (enumerableOnly) {
            symbols = symbols.filter(function(sym) {
                return Object.getOwnPropertyDescriptor(object, sym).enumerable;
            });
        }
        keys.push.apply(keys, symbols);
    }
    return keys;
}
function _object_spread_props(target, source) {
    source = source != null ? source : {};
    if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
        ownKeys(Object(source)).forEach(function(key) {
            Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
    }
    return target;
}
_colors.default.enable();
const defaultOptions = {
    filter: /\.(css|scss|sass|less)(\?.*)?$/,
    cssModules: {
        pattern: '[local]__[hash]'
    },
    browsers: '> 0.25%, not dead'
};
const styleLoader = (options = {})=>{
    const opts = (0, _deepmerge.default)(defaultOptions, options);
    const targets = (0, _utils.generateTargets)(opts.browsers);
    const allNamespaces = Array.from(new Set([
        'file'
    ].concat(opts.namespace || [])));
    const getLogger = (build)=>{
        const { logLevel } = build.initialOptions;
        if (logLevel === 'debug' || logLevel === 'verbose') {
            return (...args)=>{
                console.log('[esbuild-style-loader]'.magenta.bold, ...args);
            };
        }
        return ()=>void 0;
    };
    return {
        name: 'style-loader',
        setup (build) {
            const buildOptions = build.initialOptions;
            const logger = getLogger(build);
            const cwd = process.cwd();
            const styleTransform = function() {
                var _ref = _async_to_generator(function*(filePath) {
                    const extname = _nodepath.default.extname(filePath);
                    if (extname === '.less') {
                        return yield (0, _transformless.transformLess)(filePath, {
                            sourcemap: !!buildOptions.sourcemap,
                            alias: buildOptions.alias
                        }).catch((error)=>{
                            logger(`transform less error: ${filePath}`.red.bold);
                            logger(error);
                            throw (0, _lessutils.convertLessError)(error);
                        });
                    }
                    if (extname === '.styl') {
                        // TODO: support stylus
                        throw new Error('stylus is not supported yet');
                    }
                    if (extname === '.scss' || extname === '.sass') {
                        return yield (0, _transformsass.transformSass)(filePath, {
                            sourcemap: !!buildOptions.sourcemap,
                            alias: buildOptions.alias
                        }).catch((error)=>{
                            logger(`transform sass error: ${filePath}`.red.bold);
                            logger(error);
                            throw (0, _sassutils.convertScssError)(error, filePath);
                        });
                    }
                    const code = yield (0, _promises.readFile)(filePath, 'utf-8');
                    return {
                        css: code,
                        map: ''
                    };
                });
                return function styleTransform(filePath) {
                    return _ref.apply(this, arguments);
                };
            }();
            const handleResolve = function() {
                var _ref = _async_to_generator(function*(args) {
                    const { path: fullPath, query } = yield (0, _utils.resolvePath)(args, build);
                    return {
                        path: fullPath,
                        namespace: 'style-loader',
                        pluginData: {
                            rawPath: args.path,
                            query: _querystring.default.parse(query),
                            resolveDir: args.resolveDir
                        }
                    };
                });
                return function handleResolve(args) {
                    return _ref.apply(this, arguments);
                };
            }();
            /**
       * you can use `namespace` to control the order of the plugins
       */ for (const namespace of allNamespaces){
                build.onResolve({
                    filter: opts.filter,
                    namespace
                }, handleResolve);
            }
            build.onLoad({
                filter: /.*/,
                namespace: 'style-loader'
            }, function() {
                var _ref = _async_to_generator(function*(args) {
                    let cssContent;
                    let cssSourceMap;
                    let watchImports = [];
                    const pluginData = args.pluginData;
                    let entryContent = (0, _utils.cssExportsToJs)({}, pluginData.rawPath);
                    // enable css modules
                    // 1. if the file name contains `.modules.` or `.module.`
                    // 2. if the query contains `modules`
                    const styleFile = args.path;
                    const enableCssModules = /\.modules?\.(css|less|sass|scss)/.test(styleFile) || 'modules' in pluginData.query;
                    let result;
                    try {
                        const t = Date.now();
                        result = yield styleTransform(styleFile);
                        logger('Compile', _nodepath.default.relative(cwd, styleFile).blue.underline, `in ${Date.now() - t}ms`);
                        cssContent = result.css;
                        cssSourceMap = result.map;
                        watchImports = result.imports;
                    } catch (error) {
                        return {
                            errors: [
                                error
                            ],
                            resolveDir: _nodepath.default.dirname(styleFile),
                            watchFiles: [
                                styleFile
                            ]
                        };
                    }
                    let transformResult;
                    const watchFiles = [
                        styleFile
                    ];
                    if (watchImports) {
                        watchFiles.push(...watchImports);
                    }
                    try {
                        const t = Date.now();
                        transformResult = (0, _lightningcss.transform)({
                            targets: targets,
                            inputSourceMap: cssSourceMap,
                            sourceMap: true,
                            filename: styleFile,
                            cssModules: enableCssModules ? opts.cssModules : false,
                            code: Buffer.from(cssContent)
                        });
                        logger(`Transform css in ${Date.now() - t}ms`);
                    } catch (error) {
                        logger(`Transform css error: ${styleFile}`.red.bold);
                        logger(error);
                        const { loc, fileName, source } = error;
                        const lines = source.split('\n');
                        const lineText = lines[loc.line - 1];
                        return {
                            errors: [
                                {
                                    text: error.message,
                                    location: {
                                        file: (0, _utils.replaceExtension)(fileName, '.css'),
                                        line: loc.line,
                                        column: loc.column,
                                        lineText
                                    }
                                }
                            ],
                            resolveDir: _nodepath.default.dirname(styleFile),
                            watchFiles
                        };
                    }
                    const { code, map, exports: exports1 } = transformResult;
                    if (buildOptions.sourcemap && map) {
                        cssContent = (0, _utils.codeWithSourceMap)(code.toString(), map.toString());
                    } else {
                        cssContent = code.toString();
                    }
                    if (exports1) {
                        entryContent = (0, _utils.cssExportsToJs)(exports1, pluginData.rawPath);
                    }
                    return {
                        contents: entryContent,
                        loader: 'js',
                        pluginName: 'style-loader',
                        resolveDir: pluginData.resolveDir,
                        watchFiles: watchFiles,
                        pluginData: _object_spread_props(_object_spread({}, pluginData), {
                            cssContent
                        }),
                        warnings: result.warnings
                    };
                });
                return function(args) {
                    return _ref.apply(this, arguments);
                };
            }());
            build.onResolve({
                filter: opts.filter,
                namespace: 'style-loader'
            }, function() {
                var _ref = _async_to_generator(function*(args) {
                    const parsedPath = (0, _utils.parsePath)(args.path);
                    const result = yield build.resolve(parsedPath.path, {
                        kind: args.kind,
                        resolveDir: args.resolveDir,
                        importer: args.importer
                    });
                    if (result.errors.length) {
                        return {
                            errors: result.errors,
                            resolveDir: _nodepath.default.dirname(args.path),
                            watchFiles: [
                                args.path
                            ]
                        };
                    }
                    return {
                        path: result.path,
                        namespace: 'css-loader',
                        pluginData: args.pluginData,
                        watchFiles: [
                            args.path
                        ]
                    };
                });
                return function(args) {
                    return _ref.apply(this, arguments);
                };
            }());
            build.onLoad({
                filter: /.*/,
                namespace: 'css-loader'
            }, function() {
                var _ref = _async_to_generator(function*(args) {
                    const pluginData = args.pluginData;
                    const { cssContent, rawPath } = pluginData;
                    return {
                        contents: cssContent,
                        loader: 'css',
                        resolveDir: _nodepath.default.dirname(args.path),
                        watchFiles: [
                            rawPath
                        ]
                    };
                });
                return function(args) {
                    return _ref.apply(this, arguments);
                };
            }());
            build.onResolve({
                filter: /^\//,
                namespace: 'css-loader'
            }, function() {
                var _ref = _async_to_generator(function*(args) {
                    if (opts.publicPath) {
                        // absolute files base on publicPath
                        return {
                            path: _nodepath.default.join(opts.publicPath, `.${args.path}`)
                        };
                    }
                    return {
                        external: true
                    };
                });
                return function(args) {
                    return _ref.apply(this, arguments);
                };
            }());
        }
    };
};
