"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    convertScssError: function() {
        return convertScssError;
    },
    fileSyntax: function() {
        return fileSyntax;
    },
    getDefaultSassImplementation: function() {
        return getDefaultSassImplementation;
    },
    resolveCanonicalize: function() {
        return resolveCanonicalize;
    }
});
const _nodepath = /*#__PURE__*/ _interop_require_default(require("node:path"));
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function getDefaultSassImplementation() {
    let sassImplPkg = 'sass';
    try {
        require.resolve('sass');
    } catch (ignoreError) {
        try {
            require.resolve('node-sass');
            sassImplPkg = 'node-sass';
        } catch (_ignoreError) {
            try {
                require.resolve('sass-embedded');
                sassImplPkg = 'sass-embedded';
            } catch (__ignoreError) {
                sassImplPkg = 'sass';
            }
        }
    }
    return require(sassImplPkg);
}
function fileSyntax(filename) {
    if (filename.endsWith('.scss')) {
        return 'scss';
    }
    return 'indented';
}
function resolveCanonicalize(importer, file, alias) {
    const importerExt = _nodepath.default.extname(importer);
    const fileExt = _nodepath.default.extname(file);
    if (!fileExt) {
        file += importerExt;
    }
    // absolute path
    if (_nodepath.default.isAbsolute(file)) {
        return file;
    }
    const scope = file.split('/')[0];
    // alias
    if (alias[scope]) {
        const baseFile = file.slice(scope.length + 1);
        return _nodepath.default.resolve(alias[scope], baseFile);
    }
    return _nodepath.default.resolve(_nodepath.default.dirname(importer), file);
}
function convertScssError(error, filePath) {
    const [message, _, lineText, ...rest] = error.message.split('\n');
    const stack = rest[rest.length - 1];
    const lineColumn = stack.match(/(\d+):(\d+)/);
    return {
        text: message,
        location: {
            file: filePath,
            line: Number(lineColumn[1]),
            column: Number(lineColumn[2]),
            lineText: lineText.trim().replace(/\d+\s│/, '')
        }
    };
}
