"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "transformSass", {
    enumerable: true,
    get: function() {
        return transformSass;
    }
});
const _promises = require("node:fs/promises");
const _sassutils = require("./sass-utils.cjs");
const _nodefs = /*#__PURE__*/ _interop_require_wildcard(require("node:fs"));
const _nodepath = /*#__PURE__*/ _interop_require_default(require("node:path"));
const _nodeurl = require("node:url");
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
let sassEngine;
const transformSass = function() {
    var _ref = _async_to_generator(function*(filePath, options) {
        const { sourcemap } = options;
        if (!sassEngine) {
            try {
                sassEngine = (0, _sassutils.getDefaultSassImplementation)();
            } catch (e) {
                console.error(e);
                process.exit(1);
            }
        }
        const syntax = (0, _sassutils.fileSyntax)(filePath);
        const basedir = _nodepath.default.dirname(filePath);
        const contents = yield (0, _promises.readFile)(filePath, 'utf-8');
        const warnings = [];
        const alias = options.alias || {};
        const result = yield sassEngine.compileStringAsync(contents, {
            sourceMapIncludeSources: true,
            sourceMap: sourcemap,
            syntax,
            alertColor: false,
            logger: {
                warn (message, options) {
                    if (!options.span) {
                        warnings.push({
                            text: `sass warning: ${message}`
                        });
                    } else {
                        var _options_span_url;
                        var _options_span_url_pathname;
                        const filename = (_options_span_url_pathname = (_options_span_url = options.span.url) === null || _options_span_url === void 0 ? void 0 : _options_span_url.pathname) !== null && _options_span_url_pathname !== void 0 ? _options_span_url_pathname : _nodepath.default;
                        const esbuildMsg = {
                            text: message,
                            location: {
                                file: filename,
                                line: options.span.start.line,
                                column: options.span.start.column,
                                lineText: options.span.text
                            },
                            detail: {
                                deprecation: options.deprecation,
                                stack: options.stack
                            }
                        };
                        warnings.push(esbuildMsg);
                    }
                }
            },
            importer: {
                load (canonicalUrl) {
                    const pathname = (0, _nodeurl.fileURLToPath)(canonicalUrl);
                    const contents = _nodefs.readFileSync(pathname, 'utf8');
                    return {
                        contents,
                        syntax: (0, _sassutils.fileSyntax)(pathname),
                        sourceMapUrl: sourcemap ? canonicalUrl : undefined
                    };
                },
                canonicalize (url) {
                    const urlPath = (0, _sassutils.resolveCanonicalize)(filePath, url, alias);
                    if (urlPath) {
                        return (0, _nodeurl.pathToFileURL)(urlPath);
                    }
                    return null;
                }
            }
        });
        const sourceMap = result.sourceMap;
        if (sourceMap) {
            sourceMap.sourceRoot = basedir;
            sourceMap.sources = sourceMap.sources.map((source)=>{
                return _nodepath.default.relative(basedir, source.startsWith('data:') ? filePath : (0, _nodeurl.fileURLToPath)(source));
            });
        }
        const imports = result.loadedUrls.map((url)=>(0, _nodeurl.fileURLToPath)(url));
        return {
            css: result.css,
            map: result.sourceMap ? JSON.stringify(result.sourceMap) : '',
            imports,
            warnings
        };
    });
    return function transformSass(filePath, options) {
        return _ref.apply(this, arguments);
    };
}();
