"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    codeWithSourceMap: function() {
        return codeWithSourceMap;
    },
    cssExportsToJs: function() {
        return cssExportsToJs;
    },
    generateTargets: function() {
        return generateTargets;
    },
    parsePath: function() {
        return parsePath;
    },
    replaceExtension: function() {
        return replaceExtension;
    },
    resolvePath: function() {
        return resolvePath;
    }
});
const _nodepath = /*#__PURE__*/ _interop_require_default(require("node:path"));
const _browserslist = /*#__PURE__*/ _interop_require_default(require("browserslist"));
const _camelcase = require("camel-case");
const _lightningcss = require("lightningcss");
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const codeWithSourceMap = (code, map)=>{
    return `${code}/*# sourceMappingURL=data:application/json;base64,${Buffer.from(map).toString('base64')} */`;
};
const cssExportsToJs = (exports1, entryFile)=>{
    const keys = Object.keys(exports1).sort();
    const values = keys.map((key)=>exports1[key]);
    let variablesCode = '';
    const exportCode = [];
    const defaultCode = [];
    keys.forEach((key, index)=>{
        const clsVar = `s_${(0, _camelcase.camelCase)(key)}`;
        const clsSelector = values[index].name;
        variablesCode += `var ${clsVar} = "${clsSelector}";\n`;
        exportCode.push(`exports['${key}'] = ${clsVar};`);
        defaultCode.push(`'${key}':${clsVar}`);
    });
    const code = `${variablesCode};\n${exportCode.join('\n')};\nmodule.exports = {${defaultCode.join(',')}};\n`;
    if (entryFile) {
        return `require('${entryFile}');\n${code}`;
    }
    return code;
};
const parsePath = (path)=>{
    const queryIndex = path.indexOf('?');
    if (queryIndex === -1) {
        return {
            path: path,
            query: ''
        };
    }
    return {
        path: path.slice(0, queryIndex),
        query: path.slice(queryIndex + 1)
    };
};
function resolvePath(args, build) {
    return _resolvePath.apply(this, arguments);
}
function _resolvePath() {
    _resolvePath = _async_to_generator(function*(args, build) {
        const { path, query } = parsePath(args.path);
        let absolutePath = path;
        if (!_nodepath.default.isAbsolute(absolutePath)) {
            const result = yield build.resolve(absolutePath, {
                resolveDir: args.resolveDir,
                importer: args.importer,
                kind: args.kind
            });
            absolutePath = result.path;
        }
        return {
            path: absolutePath,
            query
        };
    });
    return _resolvePath.apply(this, arguments);
}
const generateTargets = (queries)=>{
    return (0, _lightningcss.browserslistToTargets)((0, _browserslist.default)(queries));
};
const replaceExtension = (file, ext)=>{
    const extName = _nodepath.default.extname(file);
    return file.slice(0, file.length - extName.length) + ext;
};
